# Ubuntu CI Services
# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import json
import logging

from restish import http, resource

from bsbuilder import amqp_utils, utils

log = logging.getLogger(__name__)


def _status():
    return utils.json_ok({
        'rabbit_configured': amqp_utils.get_config() is not None,
    })


def _build_source(source_packages, ppa, progress_trigger):
    msg = json.dumps({
        'source_packages': source_packages,
        'ppa': ppa,
        'progress_trigger': progress_trigger,
    })
    r = amqp_utils.send('bsbuilder', msg)
    if r:
        # send only returns something if it an error message
        r = http.service_unavailable(body=r)
    return r


class API(resource.Resource):
    @resource.child()
    def status(self, request, segments):
        log.debug('[status]: %s %s', request.url, str(segments))
        return utils.http_get_resource(_status)

    @resource.child()
    def build_source(self, request, segments):
        return utils.http_post_resource(_build_source)
