# Ubuntu CI Services
# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import subprocess


def assert_root():
    if os.geteuid() != 0:
        print('This command option must be run as root.')
        exit(1)


def assert_postgres():
    if not os.path.exists('/usr/bin/createdb'):
        print('postgres must be installed manually')
        exit(1)


def dbuser_exists(dbconfig):
    args = [
        'sudo', '-u', 'postgres', 'psql', '-c',
        'SELECT usename FROM pg_user WHERE usename=\'%s\';' % dbconfig['USER'],
    ]
    output = subprocess.check_output(args)
    return '(1 row)' in output


def db_exists(dbconfig):
    args = [
        'sudo', '-u', 'postgres', 'psql', '-c',
        'SELECT datname FROM pg_database WHERE datname=\'%s\'' % (
            dbconfig['NAME'])
    ]
    output = subprocess.check_output(args)
    return '(1 row)' in output


def create_user(dbconfig):
    args = [
        'sudo', '-u', 'postgres', 'createuser',
        '--no-createdb', '--encrypted', '--login', '--no-superuser',
        '--no-createrole', '--no-password', dbconfig['USER']
    ]
    subprocess.check_call(args)

    # Set a password for our new user
    args = [
        'sudo', '-u', 'postgres', 'psql', '--quiet', '-c',
        'ALTER USER \"%s\" WITH PASSWORD \'%s\'' % (
            dbconfig['USER'], dbconfig['PASSWORD'])
    ]
    subprocess.check_call(args)


def _create(dbconfig):
    args = [
        'sudo', '-u', 'postgres', 'createdb',
        '--owner=%s' % dbconfig['USER'],
        '--template=template0', '--no-password', dbconfig['NAME']
    ]
    subprocess.check_call(args)


def create_db(dbconfig):
    if dbconfig['ENGINE'] != 'django.db.backends.postgresql_psycopg2':
        print('Unsupported db engine: %s' % dbconfig['ENGINE'])
        return 1

    assert_root()
    assert_postgres()

    if dbuser_exists(dbconfig):
        print('user already exists, skipping creation')
        print('  user can be removed with: dropuser %s' % dbconfig['USER'])
    else:
        create_user(dbconfig)

    if db_exists(dbconfig):
        print('db already exists')
        print ('  db can be removed with: dropdb %s' % dbconfig['NAME'])
    else:
        _create(dbconfig)
