# Ubuntu CI Services
# Copyright 2012-2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from tastypie.authentication import ApiKeyAuthentication
from tastypie.authorization import DjangoAuthorization
from tastypie.resources import ModelResource

# don't allow deletion of objects with this API
ALLOWED_METHODS = ['get', 'post', 'patch']


class WriteAuthentication(ApiKeyAuthentication):
    '''only require authentication for non-GET operations.'''
    def is_authenticated(self, req, **kwargs):
        if req.method == 'GET':
            return True
        return super(WriteAuthentication, self).is_authenticated(req, **kwargs)


class WriteAuthorization(DjangoAuthorization):
    '''only require authorization for non-GET operations.'''

    def is_authorized(self, req, object=None):
        if req.method == 'GET':
            return True
        return super(WriteAuthorization, self).is_authorized(req, object)


class AuthorizedResource(ModelResource):
    '''Provide a common base class for authorized REST API's

    Authentication and authorization will be managed by this base class so
    other code gets the security it needs transparently.

    This may be extended by doing something like:
      class MyResource(AuthorizedResource):
          class Meta(AuthorizedResource.Meta):
              filtering = {
              }
    '''

    class Meta:
        authentication = WriteAuthentication()
        authorization = WriteAuthorization()
        allowed_methods = ALLOWED_METHODS
