# Ubuntu CI Services
# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import json

from django.contrib.auth.models import User

from tastypie.models import ApiKey
from tastypie.test import ResourceTestCase, TestApiClient


class TastypieTestCase(ResourceTestCase):
    '''A base class for RESTFul services.'''

    def setUp(self, resource_base):
        '''Set up an admin user that can make POST/PATCH calls.'''
        self.resource_base = resource_base
        if resource_base[-1] == '/':
            self.resource_base = resource_base[:-1]

        # create an api key for "post" operations
        u = User.objects.create(username='admin', is_superuser=True)
        k = ApiKey.objects.create(user=u, key='key')
        self.auth = 'ApiKey %s:%s' % (u.username, k.key)
        self.client = TestApiClient()

    def _resource(self, resource):
        if resource[0] == '/' or resource.startswith('http://'):
            # assume the caller is passing the full path
            return resource
        return self.resource_base + '/' + resource

    def get(self, resource, params={}):
        '''Get the resource and return resource as dict.'''
        resource = self._resource(resource)
        return json.loads(self.client.get(resource, data=params).content)

    def post(self, resource, params):
        '''Create the resource and return location of the new object.'''
        resource = self._resource(resource)
        resp = self.client.post(
            resource, data=params, authentication=self.auth)
        self.assertEqual(201, resp.status_code)
        return resp['location']

    def patch(self, resource, params):
        '''Update an existing resource.'''
        resource = self._resource(resource)
        resp = self.client.patch(
            resource, data=params, authentication=self.auth)
        self.assertEqual(202, resp.status_code)

    def getResource(self, resource, params={}):
        '''Grab a single resource and return it as a dict.'''
        obj = self.get(resource, params)
        if params:
            self.assertEqual(1, len(obj['objects']))
            obj = obj['objects'][0]
        return obj

    def createResource(self, resource, params):
        '''Create a resource, get it, and return the dict.'''
        loc = self.post(resource, params)
        return self.getResource(loc)
