# Ubuntu CI Services
# Copyright 2012-2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from tastypie.api import Api
from tastypie.http import HttpAccepted, HttpApplicationError
from tastypie.resources import ALL

from ppa_assigner.models import PPA, LaunchpadError

from ci_utils.tastypie.resource import AuthorizedResource


class PPAResource(AuthorizedResource):
    class Meta(AuthorizedResource.Meta):
        queryset = PPA.objects.filter()
        resource_name = 'ppa'

        filtering = {
            'reserved': ALL,
            'lockname': ALL,
        }

    def obj_create(self, bundle, **kwargs):
        '''We don't really create an object, so override tastypie.'''
        lockname = bundle.data.get('lockname')
        clean = bundle.data.get('clean')
        bundle.obj = PPA.reserve(lockname, clean)
        return bundle

    def patch_list(self, request, **kwargs):
        if request.body == '{"populate": true}':
            try:
                # populate is a bit different than normal "patch" operations
                PPA.populate_from_launchpad()
                return HttpAccepted()
            except LaunchpadError as e:
                return HttpApplicationError(e)
        return super(PPAResource, self).patch_list(request, **kwargs)


v1_api = Api(api_name='v1')
v1_api.register(PPAResource())
