# Ubuntu Continuous Integration Engine
# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.test import TestCase
from ci_utils.tastypie.test import TastypieTestCase
from people.models import Person


def create_person(name="Chris Johnston",
                  email="chris.johnston@canonical.com", is_team=False):
    person = Person()
    person.name = name
    person.email = email
    person.is_team = is_team
    person.save()
    return person


class PersonModelTest(TestCase):

    def test_creating_a_person(self):
        person = create_person()
        people_in_database = Person.objects.all()
        self.assertEquals(len(people_in_database), 1)
        only_person_in_database = people_in_database[0]
        self.assertEquals(only_person_in_database, person)

        self.assertEquals(only_person_in_database.name, "Chris Johnston")
        self.assertEquals(only_person_in_database.email,
                          "chris.johnston@canonical.com")


class PersonResourceTest(TastypieTestCase):

    def setUp(self):
        super(PersonResourceTest, self).setUp('/api/v1')
        create_person()
        self.person_1 = Person.objects.get(name='Chris Johnston')
        self.detail_url = 'person/{0}/'.format(self.person_1.pk)

    def test_get_person_list_json(self):
        obj = self.getResource('person/')
        self.assertEqual(obj['objects'][0], {
            u'email': u'chris.johnston@canonical.com',
            u'id': self.person_1.pk,
            u'is_team': False,
            u'name': u'Chris Johnston',
            u'resource_uri': u'/api/v1/person/{0}/'.format(self.person_1.pk)
        })

    def test_get_person_detail_json(self):
        obj = self.getResource(self.detail_url)

        self.assertKeys(obj,
                        ['email', 'id', 'is_team', 'name', 'resource_uri'])
        self.assertEqual(obj['name'], 'Chris Johnston')
