# Ubuntu Continuous Integration Engine
# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.db import models
from people.models import Person
from project.models import SourcePackage


class Ticket(models.Model):
    class Meta:
        db_table = 'ticket'

    NEW = 000
    QUEUED = 100
    PENDING = 110
    COMPLETED = 1000

    PKG_BUILDING = 200
    PKG_BUILDING_WAITING = 210
    PKG_BUILDING_INPROGRESS = 220
    PKG_BUILDING_COMPLETED = 230
    PKG_BUILDING_FAILED = 240

    IMAGE_BUILDING = 300
    IMAGE_BUILDING_WAITING = 310
    IMAGE_BUILDING_INPROGRESS = 320
    IMAGE_BUILDING_COMPLETED = 330
    IMAGE_BUILDING_FAILED = 340

    IMAGE_TESTING = 400
    IMAGE_TESTING_WAITING = 410
    IMAGE_TESTING_INPROGRESS = 420
    IMAGE_TESTING_PASSED = 430
    IMAGE_TESTING_FAILED = 440

    PKG_PUBLISHING = 500
    PKG_PUBLISHING_COMPLETED = 510
    PKG_PUBLISHING_FAILED = 520

    WORKFLOW_STEPS = (
        (NEW, "New"),
        (QUEUED, "Queued"),
        (PKG_BUILDING, "Package building"),
        (IMAGE_BUILDING, "Image building"),
        (IMAGE_TESTING, "Image testing"),
        (PKG_PUBLISHING, "Package publishing"),
        (COMPLETED, "Completed"),
    )
    WORKFLOW_STEP_STATUSES = (
        (NEW, "New"),
        (PENDING, "Not started"),
        (PKG_BUILDING_WAITING, "Not started"),
        (PKG_BUILDING_INPROGRESS, "In progress"),
        (PKG_BUILDING_COMPLETED, "Completed"),
        (PKG_BUILDING_FAILED, "Failed"),
        (IMAGE_BUILDING_WAITING, "Not started"),
        (IMAGE_BUILDING_INPROGRESS, "In progress"),
        (IMAGE_BUILDING_COMPLETED, "Completed"),
        (IMAGE_BUILDING_FAILED, "Failed"),
        (IMAGE_TESTING_WAITING, "Not started"),
        (IMAGE_TESTING_INPROGRESS, "In progress"),
        (IMAGE_TESTING_PASSED, "Passed"),
        (IMAGE_TESTING_FAILED, "Failed"),
        (PKG_PUBLISHING_COMPLETED, "Completed"),
        (PKG_PUBLISHING_FAILED, "Failed"),
        (COMPLETED, "Completed"),
    )
    owner = models.ForeignKey(Person)
    title = models.CharField(max_length=4096)
    description = models.TextField()
    bug_id = models.IntegerField(null=True, blank=True)
    current_workflow_step = models.IntegerField(choices=WORKFLOW_STEPS,
                                                max_length=4096, default=000)
    status = models.IntegerField(choices=WORKFLOW_STEP_STATUSES,
                                 max_length=4096, default=000)
    base_image = models.CharField(max_length=4096)

    def __unicode__(self):
        return self.title


class SourcePackageUpload(models.Model):
    class Meta:
        db_table = 'sourcepackageupload'

    sourcepackage = models.ForeignKey(SourcePackage)
    version = models.CharField(max_length=4096)

    def __unicode__(self):
        return "{} {}".format(self.sourcepackage, self.version)


class SubTicket(models.Model):
    class Meta:
        db_table = 'subticket'

    NEW = 000
    QUEUED = 100
    PENDING = 110
    COMPLETED = 1000

    PKG_BUILDING = 200
    PKG_BUILDING_WAITING = 210
    PKG_BUILDING_INPROGRESS = 220
    PKG_BUILDING_COMPLETED = 230
    PKG_BUILDING_FAILED = 240

    WORKFLOW_STEPS = (
        (NEW, "New"),
        (QUEUED, "Queued"),
        (PKG_BUILDING, "Package building"),
        (COMPLETED, "Completed"),
    )
    WORKFLOW_STEP_STATUSES = (
        (NEW, "New"),
        (QUEUED, "Queued"),
        (PKG_BUILDING_WAITING, "Not started"),
        (PKG_BUILDING_INPROGRESS, "In progress"),
        (PKG_BUILDING_COMPLETED, "Completed"),
        (PKG_BUILDING_FAILED, "Failed"),
    )
    current_workflow_step = models.IntegerField(choices=WORKFLOW_STEPS,
                                                max_length=4096, default=000)
    status = models.IntegerField(choices=WORKFLOW_STEP_STATUSES,
                                 max_length=4096, default=000)
    ticket = models.ForeignKey(Ticket)
    assignee = models.ForeignKey(Person)
    source_package_upload = models.ForeignKey(SourcePackageUpload)

    def __unicode__(self):
        return "{} {}".format(self.ticket, self.sourcepackageupload)


class Artifact(models.Model):
    class Meta:
        db_table = 'artifact'

    ARTIFACT_TYPE = (
        ('RESULTS', 'Test Results'),
        ('SPU', 'Source Package Upload'),
        ('LOGS', 'Logs'),
    )
    type = models.CharField(choices=ARTIFACT_TYPE, max_length=4096)
    ticket_component = models.ForeignKey(SubTicket, blank=True, null=True)
    # 'reference' provided by the artifact manager
    reference = models.CharField(max_length=4096)
    name = models.CharField(max_length=4096)

    def __unicode__(self):
        return "{} {}".format(self.name, self.type)
