# Ubuntu Continuous Integration Engine
# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.test import TestCase
from people.models import Person
from project.models import SourcePackage
from ticket.models import Ticket, SubTicket, SourcePackageUpload, Artifact


def create_person(name='Chris Johnston', email='chrisjohnston@ubuntu.com'):
    person = Person()
    person.name = name
    person.email = email
    person.save()
    return person


def create_sourcepackage(name='my-package'):
    sourcepackage = SourcePackage()
    sourcepackage.name = name
    sourcepackage.save()
    return sourcepackage


def create_sourcepackageupload(sourcepackage='my-package', version='1.1'):
    spu = SourcePackageUpload()
    spu.sourcepackage = sourcepackage
    spu.version = version
    spu.save()
    return spu


def create_ticket(title="This is my ticket",
                  description="this is my ticket description",
                  bug_id='12345', owner='Chris Johnston',
                  current_workflow_step=Ticket.PKG_BUILDING,
                  status=Ticket.PKG_BUILDING_INPROGRESS, base_image='17'):
    ticket = Ticket()
    ticket.owner = owner
    ticket.title = title
    ticket.description = description
    ticket.bug_id = bug_id
    ticket.current_workflow_step = current_workflow_step
    ticket.status = status
    ticket.base_image = base_image
    ticket.save()
    return ticket


def create_subticket(current_workflow_step=SubTicket.PKG_BUILDING, ticket=None,
                     status=SubTicket.PKG_BUILDING_INPROGRESS,
                     assignee='Chris Johnston',
                     source_package_upload=None):
    subticket = SubTicket()
    subticket.current_workflow_step = current_workflow_step
    subticket.status = status
    subticket.ticket = ticket
    subticket.assignee = assignee
    subticket.source_package_upload = source_package_upload
    subticket.save()
    return subticket


def create_artifact(name="test.log", ticket_component=None,
                    reference='123abc'):
    artifact = Artifact()
    artifact.name = name
    artifact.ticket_component = ticket_component
    artifact.reference = reference
    artifact.save()
    return artifact


class TicketModelsTestCase(TestCase):
    def setUp(self):
        self.person = create_person()
        self.sourcepackage = create_sourcepackage()
        self.spu = create_sourcepackageupload(
            sourcepackage=self.sourcepackage)
        self.ticket = create_ticket(owner=self.person)
        self.subticket = create_subticket(ticket=self.ticket,
                                          assignee=self.person,
                                          source_package_upload=self.spu)
        self.artifact_1 = create_artifact(ticket_component=self.subticket)

    def test_creating_an_artifact(self):
        artifact_in_database = Artifact.objects.exclude(
            ticket_component=None)
        self.assertEquals(len(artifact_in_database), 1)
        only_artifact_in_database = artifact_in_database[0]
        self.assertEquals(only_artifact_in_database, self.artifact_1)

        self.assertEquals(only_artifact_in_database.name, "test.log")

    def test_creating_a_ticket(self):
        ticket_in_database = Ticket.objects.all()
        self.assertEquals(len(ticket_in_database), 1)
        only_ticket_in_database = ticket_in_database[0]
        self.assertEquals(only_ticket_in_database, self.ticket)

        self.assertEquals(only_ticket_in_database.title, "This is my ticket")

    def test_creating_a_subticket(self):
        subticket_in_database = SubTicket.objects.all()
        self.assertEquals(len(subticket_in_database), 1)
        only_subticket_in_database = subticket_in_database[0]
        self.assertEquals(only_subticket_in_database, self.subticket)

        self.assertEquals(only_subticket_in_database.assignee.name,
                          "Chris Johnston")

    def test_creating_a_sourcepackageupload(self):
        spu_in_database = SourcePackageUpload.objects.all()
        self.assertEquals(len(spu_in_database), 1)
        only_spu_in_database = spu_in_database[0]
        self.assertEquals(only_spu_in_database, self.spu)

        self.assertEquals(only_spu_in_database.sourcepackage.name,
                          "my-package")
